@@ -1,5 +1,4 @@
 import logging; logger = logging.getLogger("minimalKB."+__name__);
-DEBUG_LEVEL=logging.DEBUG
 
 from Queue import Queue, Empty
 import json
@@ -317,7 +316,7 @@ class MinimalKB:
             lifespan = policy.get('lifespan', 0)
 
             logger.info("Adding to " + str(list(models)) + ":\n\t- " + "\n\t- ".join([str(s) for s in stmts]) + \
-                    " (lifespan: %ssec)"%lifespan if lifespan else "")
+                    (" (lifespan: %ssec)"%lifespan if lifespan else ""))
             for model in models:
                 self.store.add(stmts, model, lifespan=lifespan)
 
@@ -331,7 +330,7 @@ class MinimalKB:
             lifespan = policy.get('lifespan', 0)
             
             logger.info("Updating " + str(list(models)) + " with:\n\t- " + "\n\t- ".join([str(s) for s in stmts]) + \
-                    " (lifespan: %ssec)"%lifespan if lifespan else "")
+                    (" (lifespan: %ssec)"%lifespan if lifespan else ""))
             for model in models:
                 self.store.update(stmts, model, lifespan=lifespan)
 
@@ -473,10 +472,13 @@ class MinimalKB:
         models = self.normalize_models(models)
         return self.store.classesof(concept, direct, models)
 
-    @compat
     @api
     def close(self):
-        pass
+        """ This code should actually never be called: the 'close' request
+        is intercepted at the front-end level (eg, socket server) to close
+        the connection.
+        """
+        raise NotImplementedError
 
     ################################################################################
     ################################################################################
@@ -486,7 +488,7 @@ class MinimalKB:
                 clients = self.eventsubscriptions[e.id]
                 logger.info("Event %s triggered. Informing %s clients." % (e.id, len(clients)))
                 for client in clients:
-                    msg = "event\n%s\n%s\n" % (e.id, json.dumps(e.content, ensure_ascii=False).encode("utf8"))
+                    msg = ("event", e)
                     self.requestresults.setdefault(client,Queue()).put(msg)
                 if not e.valid:
                     self.active_evts.discard(e)
@@ -498,6 +500,13 @@ class MinimalKB:
         self._lifespan_manager = Process(target = lifespan.start_service, args = ('kb.db',))
         self._lifespan_manager.start()
 
+    def stop_services(self):
+        self._reasoner.terminate()
+        self._lifespan_manager.terminate()
+
+        self._reasoner.join()
+        self._lifespan_manager.join()
+
     def normalize_models(self, models):
         """ If 'models' is None, [] or contains 'all', then
         returns the set of all models known to the KB.
@@ -516,37 +525,45 @@ class MinimalKB:
         else:
             return self.models
 
-    def execute(self, client, name, *args):
+    def execute(self, client, name, *args, **kwargs):
+
+        if name == "close":
+            logger.info("Closing connection to client.")
+            return
+
         f = getattr(self, name)
         if hasattr(f, "_compat"):
                 logger.warn("Using non-standard method %s. This may be " % f.__name__ + \
                         "removed in the future!")
         
-        msg = ""
+        msg = None
         try:
             res = None
-            if args:
-                res = f(*args)
+            if args or kwargs:
+                res = f(*args, **kwargs)
                 if name=="subscribe":
                     self.eventsubscriptions.setdefault(res, []).append(client)
             else:
                 res = f()
-            msg = "ok\n%s\n" % json.dumps(res, ensure_ascii=False).encode("utf8") if res is not None else "ok\n"
+            msg = ("ok", res)
         except Exception as e:
             logger.debug(traceback.format_exc())
-            logger.error("Request failed: %s" % e)
-            msg = "error\nKbError\n%s\n" % str(e)
+            logger.error("request failed: %s" % e)
+            msg = ("error", e)
 
         self.requestresults.setdefault(client,Queue()).put(msg)
 
-    def submitrequest(self, client, name, *args):
-        self.incomingrequests.put((client, name, args))
+    def submitrequest(self, client, name, *args, **kwargs):
+        self.incomingrequests.put((client, name, args, kwargs))
 
     def process(self):
         try:
-            client, name, args = self.incomingrequests.get(True, 0.05)
-            logger.debug("Processing now %s%s" %(name, str(args)))
-            self.execute(client, name, *args)
+            client, name, args, kwargs = self.incomingrequests.get(True, 0.05)
+            logger.debug("Processing <%s(%s,%s)>..." % \
+                            (name, 
+                             ", ".join([str(a) for a in args]),
+                             ", ".join(str(k)+"="+str(v) for k,v in kwargs.items())))
+            self.execute(client, name, *args, **kwargs)
         except Empty:
             pass
 
